/*
 * DetectChange V4.0
 * Author P.Peters
 * Date: 20171005
 * 
 * Detect quick light change using an LDR, allowing for slow light changes 
 * 
 */

// LDR is connected to pin A0 and GND
#define sensorPin A0

// LED connected to pin 13 (builtin)
#define ledPin LED_BUILTIN

// storage for sensor value read
int sensorValue = 0;

// average light level measured before
float avg;

void setup() {
  // define I/O pins
  pinMode(ledPin, OUTPUT);
  pinMode(sensorPin, INPUT_PULLUP);
  // allow for 'monitoring'
  Serial.begin(115200);
  //read an initial sensor value to have a start average value
  avg = (float)analogRead(sensorPin);
}

// calculate new average from previous average and new sensor value
float calculateNewAverage(int newval){
  // this simulates an array of sensor values that holds 20000 samples
  return (19999 * avg + newval)/20000; 
}

void loop() {
  // read the value from the sensor:
  sensorValue = analogRead(sensorPin);
  // calculate new average
  avg = calculateNewAverage(sensorValue);
  // for monitoring purposes output the sensor and average values
  Serial.print(sensorValue);
  Serial.print("; ");
  Serial.println(avg);
  
  // do something with the value read from the sensor
  // the -30 in avg-30 below indicates how much deviation from the average 
  // is needed to trigger an action (the 'sensitivity')
  
  if (sensorValue < avg-30) {
    // switch LED ON
    digitalWrite(ledPin, HIGH);
  } else {
    // switch LED off
    digitalWrite(ledPin, LOW);
  }
}
