/*
   Use processing to control the color of a neopixel strip
   color is received as a series of 3 integer values represented as
   text and separated by commas on a single line

   Author: P.Peters

   Derived from Adafruit neopixel library example

   Version 0.4
   2017-09-13
*/

#include <Adafruit_NeoPixel.h>

#define NEOPIN 6   // Neopixel data pin coonected to tghs pin
#define NEOLEDS 8  // Number of leds on neopixel strand
#define LF 10      // end of line character

Adafruit_NeoPixel strip = Adafruit_NeoPixel(NEOLEDS, NEOPIN, NEO_RGB + NEO_KHZ800);

// brightness of the neopixel leds in %
int brightness = 50;
// color values; RGB and intensity
int rVal = 0, gVal = 0, bVal = 0, iVal = 0;
// data received or not
boolean dataValid = false;
// placeholder for received data
String inputString;

void setup() {
  // for debugging purposes only
  Serial.begin(115200);
  // initialise strip
  strip.begin();
  strip.show(); // Initialize all pixels to 'off'
}

void loop() {
  // if a new command comes in, process it
  if (dataValid) {
    // wait for new data when this is processed
    dataValid = false;
    // set brightness and color as defined by the received data
    strip.setBrightness(iVal);
    shiftInColor(strip.Color(gVal, rVal, bVal));
  }
}

// first led gets color c, the rest shifts up one led
void shiftInColor(uint32_t c) {
  for (int i = strip.numPixels(); i > 0; i--) strip.setPixelColor(i, strip.getPixelColor(i - 1));
  strip.setPixelColor(0, c);
  strip.show();
}

// here is where serial data is received
void serialEvent() {
  // any data?
  while (Serial.available()) {
    // read and process it
    char ch = Serial.read();
    // new line indicates command is complete
    if (ch == '\n') {
      // strip leading and trailing spaces and newline
      inputString.trim();
      // for debugging, data is sent back
      Serial.println(inputString);
      // get all the parts delimted by commas
      // note, if an invalid comand arrives this is not handled
      if (inputString.indexOf(',') > 0) {
        // only if commas are there it's valid
        rVal = inputString.substring(0, inputString.indexOf(',')).toInt();
        //Serial.println(rVal);
        inputString = inputString.substring(inputString.indexOf(',') + 1);
        gVal = inputString.substring(0, inputString.indexOf(',')).toInt();
        //Serial.println(gVal);
        inputString = inputString.substring(inputString.indexOf(',') + 1);
        bVal = inputString.substring(0, inputString.indexOf(',')).toInt();
        //Serial.println(bVal);
        inputString = inputString.substring(inputString.indexOf(',') + 1);
        iVal = inputString.toInt();
      } else {
        // no commas, make it white
        rVal = 255; bVal = 255; gVal = 255, iVal = 50;
      }
      // mark that data was received
      dataValid = true;
      // and clear the processed data string
      inputString = "";
    } else {
      // no new line yet, add character to input string
      inputString = inputString + ch;
    }
  }
}


